import hashlib
import logging
from enum import Enum
from io import BytesIO
from pathlib import Path
from typing import BinaryIO, List, Optional, Tuple

import requests
from aideate_scraper.core.s3_utils import parse_s3_url, r2_manager
from botocore.exceptions import ClientError
from httpx import AsyncClient, Client
from PIL import Image

from settings import HTTP_MEDIA_ASSETS, IMAGE_ASSETS_DIR, S3_MEDIA_ASSETS

logger = logging.getLogger(__name__)

ALLOWED_EXTENSIONS = ["png", "jpeg", "jpg", "webp"]


def load_image_from_dir(folder_path: str) -> List[Image.Image]:
    path = Path(folder_path)
    assert path.is_dir()
    return [Image.open(p).copy() for p in path.iterdir()]


def parse_signature_from_path(image_path: str) -> str:
    return image_path.split("/")[-1].split(".")[0]


class Resolution(Enum):
    ORIGINAL = "original"
    RES_345x = "345x"


def resize_image(img: Image.Image, resolution: Resolution) -> Image.Image:
    if resolution == Resolution.RES_345x:
        base_width = 345
        w_percent = base_width / float(img.size[0])
        h_size = int((float(img.size[1]) * float(w_percent)))
        return img.resize((base_width, h_size))
    else:
        raise ValueError("Unsupported resolution")


def http_path_from_signature(
    md5: str,
    resolution: Resolution = Resolution.ORIGINAL,
    save_dir: str = IMAGE_ASSETS_DIR,
) -> str:
    if resolution == Resolution.ORIGINAL:
        return HTTP_MEDIA_ASSETS + str(
            Path(save_dir, md5[0:2], md5[2:4], md5[4:6], md5 + ".jpg")
        )
    else:
        return HTTP_MEDIA_ASSETS + str(
            Path(save_dir, resolution.value, md5[0:2], md5[2:4], md5[4:6], f"{md5}.jpg")
        )


def get_content_signature(file_bytes: bytes):
    md5 = hashlib.md5(file_bytes)
    return md5.hexdigest()


def s3_path_from_signature(
    md5: str,
    resolution: Resolution = Resolution.ORIGINAL,
    save_dir: str = IMAGE_ASSETS_DIR,
) -> str:
    if resolution == Resolution.ORIGINAL:
        return S3_MEDIA_ASSETS + str(
            Path(save_dir, md5[0:2], md5[2:4], md5[4:6], md5 + ".jpg")
        )
    else:
        return S3_MEDIA_ASSETS + str(
            Path(save_dir, resolution.value, md5[0:2], md5[2:4], md5[4:6], f"{md5}.jpg")
        )


def http_image_path_from_s3_path(s3_path: str) -> str:
    return s3_path.replace(S3_MEDIA_ASSETS, HTTP_MEDIA_ASSETS)


def s3_path_from_http_image_path(s3_path: str) -> str:
    return s3_path.replace(HTTP_MEDIA_ASSETS, S3_MEDIA_ASSETS)


def allowed_img_file(filename):
    return "." in filename and filename.rsplit(".", 1)[1].lower() in ALLOWED_EXTENSIONS


class ImageManager(object):
    def s3_put_uploaded_image(
        self, file_obj: BinaryIO, filename: str, return_http=True
    ) -> Optional[str]:
        """
        Mostly just some extra protection against uploaded images
        """
        if file_obj and allowed_img_file(filename):
            file_bytes = file_obj.read()
            return self.s3_put_image(
                file_bytes,
                return_http=return_http,
            )
        else:
            raise ValueError("Input file was not allowed")

    def s3_put_image(
        self,
        file_bytes: bytes,
        save_dir: str = IMAGE_ASSETS_DIR,
        return_http=True,
    ) -> str:
        try:
            # convert all image formats into jpg
            img = Image.open(BytesIO(file_bytes)).convert("RGB")
            thumbnail_img = resize_image(img, Resolution.RES_345x)

            origin_b = BytesIO()
            thumbnail_b = BytesIO()
            img.save(origin_b, format="jpeg")
            thumbnail_img.save(thumbnail_b, format="jpeg")
            origin_b.seek(0)
            thumbnail_b.seek(0)
        except:
            raise ValueError("Could not save the given image file")

        md5 = get_content_signature(origin_b.getvalue())
        s3_origin_path = s3_path_from_signature(md5, save_dir=save_dir)
        s3_thumbnail_path = s3_path_from_signature(
            md5, resolution=Resolution.RES_345x, save_dir=save_dir
        )

        futures = []
        for s3_path, bytes_buffer in [
            (s3_origin_path, origin_b),
            (s3_thumbnail_path, thumbnail_b),
        ]:
            if not r2_manager.exists(s3_path):
                bucket, key = parse_s3_url(s3_path)
                try:
                    futures.append(
                        r2_manager.put_fileobj(
                            bytes_buffer,
                            bucket=bucket,
                            key=key,
                            extra_args={
                                "ACL": "public-read",
                                "ContentType": "image/jpeg",
                            },
                            blocking=False,
                        )
                    )
                except ClientError as e:
                    logging.error(e)
                    raise e
        for f in futures:
            # block
            f.result()
        http_path = http_image_path_from_s3_path(s3_origin_path)
        if return_http:
            logger.debug(f"Uploaded to: {http_path}")
            return http_path
        else:
            logger.debug(f"Uploaded to: {s3_origin_path}")
            return s3_origin_path

    def s3_get_image(self, s3_path: str) -> Image.Image:
        logger.debug(f"Reading from: {s3_path}")
        bucket, key = parse_s3_url(s3_path)

        try:
            file_obj = BytesIO(r2_manager.get(bucket, key))
            return Image.open(file_obj)
        except ClientError as e:
            logging.error(e)
            raise e

    def http_get_image(
        self, http_path: str, timeout_secs: Optional[int] = None
    ) -> Image.Image:
        try:
            img_data = requests.get(http_path, timeout=timeout_secs).content
            return Image.open(BytesIO(img_data))
        except Exception as e:
            logging.error(e)
            raise e

    async def async_http_get_image(
        self, http_path: str, timeout_secs: Optional[int] = None
    ) -> Optional[Tuple[str, Image.Image]]:
        try:
            async with AsyncClient(timeout=timeout_secs) as client:
                response = await client.get(http_path, follow_redirects=True)
                response.raise_for_status()  # Raises exception for 4XX/5XX responses
                img_data = response.content
            return http_path, Image.open(BytesIO(img_data))

        except Exception as e:
            logging.error(f"Failed to fetch or open image from {http_path}: {e}")
        return None

    def is_aideate_url(self, url: str) -> bool:
        return url.startswith(HTTP_MEDIA_ASSETS) and url.endswith("jpg")

    def get_aideate_url(self, url: str) -> Optional[str]:
        if self.is_aideate_url(url):
            return url

        try:
            img: Image.Image = image_manager.http_get_image(url)
            img_bytes = BytesIO()
            img.save(img_bytes, format=img.format)

            return image_manager.s3_put_image(
                file_bytes=img_bytes.getvalue(), return_http=True
            )
        except:
            return None

    async def async_get_aideate_url(self, url: str) -> str:
        try:
            async with AsyncClient() as client:
                img_response = await client.get(url, follow_redirects=True)
            return image_manager.s3_put_image(
                file_bytes=img_response.content, return_http=True
            )
        except Exception as e:
            logging.error(e)
            raise e


image_manager = ImageManager()


def put_image_to_s3(img: Image.Image, format="PNG"):
    """put the pil image to s3"""
    img_data = BytesIO()
    img.save(img_data, format)
    img_data.seek(0)
    return image_manager.s3_put_image(img_data.read(), return_http=True)


if __name__ == "__main__":
    import sys

    with open("ml/assets/instances/kevin/kevin2.jpg", "rb") as file:
        print(image_manager.s3_put_image(file.read(), return_http=True))

    with open("ml/assets/instances/png/among-us.png", "rb") as file:
        print(image_manager.s3_put_image(file.read(), return_http=True))

    with open("ml/assets/instances/webp/stella.webp", "rb") as file:
        print(image_manager.s3_put_image(file.read(), return_http=True))



import hashlib
import logging
from io import BytesIO
from pathlib import Path
from typing import Optional, Union

import magic
import requests
from botocore.exceptions import ClientError

from settings import DOC_ASSETS_DIR, HTTP_MEDIA_ASSETS, S3_MEDIA_ASSETS
from aideate_scraper.core.s3_utils import parse_s3_url, r2_manager

logger = logging.getLogger(__name__)


def parse_signature_from_path(document_path: str) -> str:
    return document_path.split("/")[-1].split(".")[0]


def get_content_signature(file_str: bytes):
    md5 = hashlib.md5(file_str)
    return md5.hexdigest()


def http_path_from_signature(
    md5: str,
) -> str:
    return HTTP_MEDIA_ASSETS + str(
        Path(DOC_ASSETS_DIR, md5[0:2], md5[2:4], md5[4:6], md5)
    )


def s3_path_from_signature(
    md5: str,
) -> str:
    return S3_MEDIA_ASSETS + str(
        Path(DOC_ASSETS_DIR, md5[0:2], md5[2:4], md5[4:6], md5)
    )


class DocumentManager(object):
    def s3_put_document(
        self,
        file_str: Union[str, bytes],
    ) -> str:
        """put document file to s3 and return signature"""

        if isinstance(file_str, str):
            file_str = file_str.encode("utf-8")

        # get mimetype to control content type view on s3
        content_type = magic.from_buffer(BytesIO(file_str).read(2048), mime=True)

        signature = get_content_signature(file_str)
        s3_path = s3_path_from_signature(signature)

        if not r2_manager.exists(s3_path):
            bucket, key = parse_s3_url(s3_path)

            try:
                r2_manager.put_fileobj(
                    BytesIO(file_str),
                    bucket=bucket,
                    key=key,
                    extra_args={"ACL": "public-read", "ContentType": content_type},
                )
            except ClientError as e:
                logging.error(e)
                raise e

        return signature

    def s3_get_document(self, signature: str) -> bytes:
        s3_path = s3_path_from_signature(signature)
        return r2_manager.get_from_s3_path(s3_path)

    def http_get_document(
        self, http_path: str, timeout_secs: Optional[int] = None
    ) -> bytes:
        try:
            data = requests.get(http_path, timeout=timeout_secs).content
            return data
        except Exception as e:
            logging.error(e)
            raise e

    def is_aideate_url(self, url: str) -> bool:
        return url.startswith(HTTP_MEDIA_ASSETS + str(Path(DOC_ASSETS_DIR)))


doc_manager = DocumentManager()


if __name__ == "__main__":
    pass



from subprocess import check_call

from setuptools import Command, find_packages, setup


class PublishCommand(Command):
    description = "Build and publish the package to PyPI"
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        check_call("python setup.py sdist bdist_wheel", shell=True)
        check_call("twine upload dist/*", shell=True)


setup(
    name="aideate-blade",
    version="1.0.14",
    description="web content aideate_scraper",
    author="Aideate",
    author_email="xiaye@aideate.ai",
    packages=find_packages(),
    install_requires=[
        # List your dependencies here
        "pydantic==2.6.1",
        "ipdb==0.13.11",
        "simplejson==3.17.6",
        "boto3==1.29.6",
        "boto3[crt]==1.29.6",
        "sqlalchemy==2.0.5.post1",
        # "psycopg2==2.9.5",
        "scrapetube==2.3.1",
        "overrides==7.3.1",
        "snakemd==2.1.0",
        "html2text==2020.1.16",
        "playwright==1.35.0",
        "prometheus-client==0.20.0",
        "extraction==0.3",
        "pdfplumber==0.11.0",
        "stripe==5.5.0",
        "python-magic==0.4.27",
        "StrEnum==0.4.15",
        "modal==0.64.110",
        "setuptools==69.5.1",
        "tenacity==8.3.0",
        "httpx==0.23.3",
        "wandb==0.17.0",
        "docstring_parser==0.16",
    ],
    cmdclass={
        "publish": PublishCommand,
    },
)



import io
import logging
import os
import shutil
from pathlib import Path

import boto3
import boto3.s3.transfer as s3transfer
from botocore.config import Config
from botocore.errorfactory import ClientError

from settings import R2_ACCESS_KEY_ID, R2_ENDPOINT_URL, R2_SECRET_ACCESS_KEY

logger = logging.getLogger(__name__)


def parse_s3_url(url: str):
    return url.replace("s3://", "").split("/", 1)


class S3Manager(object):
    def __init__(self, workers: int = 10, **kwargs):
        config = Config(retries=dict(max_attempts=3), max_pool_connections=64)
        self.s3_client = boto3.client("s3", config=config, **kwargs)
        transfer_config = s3transfer.TransferConfig(
            use_threads=True,  # causes some out of pool connection warnings w/ threading
            max_concurrency=workers,
        )
        self.s3t = s3transfer.create_transfer_manager(self.s3_client, transfer_config)

    def _get_file_folders(self, s3_path):
        bucket, key = parse_s3_url(s3_path)

        file_names = []
        folders = []
        default_kwargs = {"Bucket": bucket, "Prefix": key}
        next_token = ""
        prefix = key if key[-1] == "/" else key + "/"

        while next_token is not None:
            updated_kwargs = default_kwargs.copy()
            if next_token != "":
                updated_kwargs["ContinuationToken"] = next_token

            response = self.s3_client.list_objects_v2(**default_kwargs)
            contents = response.get("Contents")

            for result in contents:
                path = result.get("Key")
                assert path.startswith(prefix)
                path = path[len(prefix) :]

                if path[-1] == "/":
                    folders.append(path)
                else:
                    file_names.append(path)

            next_token = response.get("NextContinuationToken")

        return file_names, folders

    def get_recursive(self, s3_path, local_path):
        """
        https://www.learnaws.org/2022/07/02/boto3-download-files-s3/
        """

        file_names, folders = self._get_file_folders(s3_path)

        if len(file_names) + len(folders) == 0:
            raise ValueError(f"Found no files at {s3_path}")

        bucket, key = parse_s3_url(s3_path)

        local_path = Path(local_path)

        for folder in folders:
            folder_path = Path.joinpath(local_path, folder)
            folder_path.mkdir(parents=True, exist_ok=True)
            logger.info(f"mkdir -p {str(folder_path)}")

        download_futures = []
        for file_name in file_names:
            file_path = Path.joinpath(local_path, file_name)
            file_path.parent.mkdir(parents=True, exist_ok=True)
            download_future = self.s3t.download(
                bucket, str(Path(key, file_name)), str(file_path)
            )
            download_futures.append(download_future)
            logger.info(
                f"downloading s3://{bucket}/{key}/{file_name} to {str(file_path)}"
            )

        # Wait for all downloads to complete
        for future in download_futures:
            future.result()

    def exists(self, s3_path):
        bucket, key = parse_s3_url(s3_path)
        try:
            self.s3_client.head_object(Bucket=bucket, Key=key)
            return True
        except ClientError as e:
            if e.response["Error"]["Code"] == "404":
                return False
            else:
                raise

    def put_fileobj(
        self, fileobj, bucket: str, key: str, extra_args: dict, blocking: bool = True
    ):
        future = self.s3t.upload(fileobj, bucket=bucket, key=key, extra_args=extra_args)
        if blocking:
            return future.result()
        else:
            return future

    def put(self, bucket: str, key: str, content: str):
        self.s3t.upload(
            io.BytesIO(content.encode("utf-8")), bucket=bucket, key=key
        ).result()

    def put_to_s3_path(self, s3_path: str, content: str):
        bucket, key = parse_s3_url(s3_path)
        self.put(bucket, key, content)

    def get(self, bucket: str, key: str) -> bytes:
        buffer = io.BytesIO()
        self.s3t.download(bucket=bucket, key=key, fileobj=buffer).result()
        return buffer.getvalue()

    def get_from_s3_path(self, s3_path: str) -> bytes:
        bucket, key = parse_s3_url(s3_path)
        return self.get(bucket, key)

    def delete_all_in_s3_path(self, s3_path: str):
        """Delete all objects in an S3 path."""

        bucket, s3_prefix = parse_s3_url(s3_path)

        # List all objects in the s3_path
        s3_files = []
        paginator = self.s3_client.get_paginator("list_objects_v2")
        for page in paginator.paginate(Bucket=bucket, Prefix=s3_prefix):
            for obj in page.get("Contents", []):
                s3_files.append(obj["Key"])

        # Delete each file
        for s3_file in s3_files:
            self.s3t.delete(bucket=bucket, key=s3_file).result()
            print(f"Deleted {s3_file} from S3")

    def sync_to_s3(self, local_directory: str, s3_path: str, overwrite: bool = False):
        if not os.path.isdir(local_directory):
            raise ValueError("provided path is not a directory")

        if overwrite:
            # clear s3 folder before we upload our local stuff
            self.delete_all_in_s3_path(s3_path)
        else:
            if self.exists(s3_path):
                raise ValueError("Cannot overwrite s3 folder that exists")

        bucket, s3_folder = parse_s3_url(s3_path)
        for root, dirs, files in os.walk(local_directory):
            for filename in files:
                # construct the local file path
                local_path = os.path.join(root, filename)

                # construct the path on S3
                relative_path = os.path.relpath(local_path, local_directory)
                s3_path = os.path.join(s3_folder, relative_path).replace(
                    "\\", "/"
                )  # make sure we use '/' for S3

                # upload file
                try:
                    self.s3t.upload(local_path, bucket=bucket, key=s3_path).result()
                    print(f"Uploaded {local_path} to s3://{bucket}/{s3_path}")
                except Exception as e:
                    print(
                        f"Failed to upload {local_path} to s3://{bucket}/{s3_path}. Reason: {e}"
                    )

    def sync_to_local(
        self, s3_path: str, local_directory: str, overwrite: bool = False
    ):
        if os.path.exists(local_directory):
            if overwrite:
                print(f"Deleting folder: {local_directory}")
                shutil.rmtree(local_directory)
            else:
                raise ValueError("Cannot overwrite local folder that exists")
        os.makedirs(local_directory)

        bucket, s3_folder = parse_s3_url(s3_path)

        # get the list of files from the s3 folder
        s3_files = self.s3_client.list_objects(Bucket=bucket, Prefix=s3_folder)[
            "Contents"
        ]

        future_to_s3_file = {}
        for s3_file in s3_files:
            local_path = os.path.join(
                local_directory, os.path.relpath(s3_file["Key"], s3_folder)
            )
            # Create nested directories if necessary
            os.makedirs(os.path.dirname(local_path), exist_ok=True)
            future_to_s3_file[
                self.s3t.download(bucket, s3_file["Key"], local_path)
            ] = s3_file

        # Process completed tasks
        for future, s3_file in future_to_s3_file.items():
            s3_file_path = s3_file["Key"]
            local_path = os.path.join(
                local_directory, os.path.relpath(s3_file_path, s3_folder)
            )

            try:
                future.result()  # This will block until the download is done and raise an exception if the download failed
                print(f"Downloaded s3://{bucket}/{s3_file_path} to {local_path}")
            except Exception as e:
                print(
                    f"Failed to download s3://{bucket}/{s3_file_path} to {local_path}. Reason: {e}"
                )
        print("Download is done")


r2_manager = S3Manager(
    endpoint_url=R2_ENDPOINT_URL,
    aws_access_key_id=R2_ACCESS_KEY_ID,
    aws_secret_access_key=R2_SECRET_ACCESS_KEY,
    region_name="auto",
)



import argparse
import asyncio
import os
from typing import List, Optional

from pydantic import BaseModel

from aideate_scraper.core.scrape import ascrape_web, parse_html
from aideate_scraper.core.doc_utils import doc_manager, s3_path_from_signature
from aideate_scraper.core.image_utils import image_manager
from aideate_scraper.core.s3_utils import r2_manager

URL = [
    "https://arxiv.org/pdf/2307.01952.pdf",
    "https://www.youtube.com/watch?v=SKia5QUiGkE",
    "https://www.instagram.com/p/BXgUfl-FBaf/?hl=en",
    "https://www.instagram.com/p/C3iVyS1SOBO/",
    "https://www.youtube.com/shorts/o0Q6mK-hSIw",
    "https://ttlc.intuit.com/turbotax-support/en-us/help-article/taxation/prepare-rdp-return-live-california-nevada-state/L4VNsnTGJ_US_en_US?fbclid=IwAR393CPXNw6ndYF1WxTZfCMFUOwCkp5Hmm2TyQcN2B2Y7QIjRkElhQnN1zA",
    "https://www.reddit.com/r/OneTruthPrevails/",
    "https://twitter.com/vrushankdes/status/1788281555288265201",
    "https://www.facebook.com/reel/357925126811726",
]

URL_CONTENT_SIGNATURE = [
    (
        "https://www.instagram.com/p/C3iVyS1SOBO/",
        "9b4557db08617cce07658d86579a2391",
    ),
    (
        "https://twitter.com/vrushankdes/status/1788281555288265201",
        "d99c687fd0538e9bda0f95f76fd8a5d2",
    ),
    (
        "https://www.instagram.com/p/C38M64pI4p7/",
        "914012eb12318311e0ef263d4fcf3968",
    ),
    (
        "https://www.reddit.com/r/aws/comments/16g3gv1/any_experiences_with_t3micro_for_rds_postgresql/",
        "2ac7df2cdc84089c98b8322df9a3de99",
    ),
]


class ScrapeRow(BaseModel):
    url: str
    thumbnail_url: Optional[str]
    title: Optional[str]
    text: Optional[str]
    texts: Optional[List[str]]
    images: Optional[List[str]]


def render_html(
    experiment_name: str,
    scrapes: List[ScrapeRow],
    content_scrapes: List[ScrapeRow],
    outdir="/data1/experiments",
):
    if not scrapes and not content_scrapes:
        return

    def _scrape_row_to_html(_scrapes):
        # mirror into our site locally
        for _scrape in _scrapes:
            if _scrape and _scrape.thumbnail_url:
                img = image_manager.get_aideate_url(_scrape.thumbnail_url)
                if img:
                    _scrape.thumbnail_url = img

        # Create a table for the scrape data
        header_row = "<tr><th>URL</th><th>Thumbnail</th><th>Title</th><th>Text</th><th>Texts</th><th>Images</th></tr>"
        rows_html = ""
        for scrape in _scrapes:
            rows_html += f"""
<tr>
    <td style="white-space:pre-wrap; word-wrap:break-word">{scrape.url}</td>
    <td><img src="{scrape.thumbnail_url}" style="height:200px;"></td><td>{scrape.title}</td>'
    <td style="white-space:pre-wrap; word-wrap:break-word" onclick="toggleExpandableContent(this)">
        Click to expand
        <div class="expandable-content">
            {scrape.text}
        </div>
    </td>"""
            if scrape.texts:
                rows_html += '<td style="white-space:pre-wrap; word-wrap:break-word">' + "".join(f"{text}" for text in scrape.texts) + "</td>"
            else:
                rows_html += "<td></td>"
            rows_html += "<td>"
            for image in scrape.images or []:
                if image:
                    rows_html += f'<div><img src="{image}" style="height:200px; width:200px;"></div>'
            rows_html += '</td></tr>'

        table_html = f"<table border='1'>{header_row}{rows_html}</table>"
        return table_html

    style = """
<style>
    .expandable-content {
        display: none; 
    }
</style>
    """
    script = """
<script>
    // Function to toggle visibility of expandable content
    function toggleExpandableContent(td) {
        var content = td.querySelector('.expandable-content');
        if (content.style.display === 'none') {
            content.style.display = 'block'; // Show the content
        } else {
            content.style.display = 'none'; // Hide the content
        }
    }
</script>
    """

    output = f""""
<html>
    <head>
        <meta charset="UTF-8">
        <meta name="viewport" content="width=device-width, initial-scale=1.0">
        <title>Expandable Table Cell</title>
        {style}
        {script}
    </head>
    <body>
    {_scrape_row_to_html(scrapes)}<br>
    {_scrape_row_to_html(content_scrapes)}
    </body>
</html>"""
    output_file = f"{outdir}/{experiment_name}.html"
    if not os.path.exists(outdir):
        os.makedirs(outdir)
    with open(output_file, "w") as f:
        f.write(output)

    # Log to wandb as an artifact
    artifact = wandb.Artifact(f"{experiment_name}.html", type="file")
    artifact.add_file(output_file)
    wandb.log_artifact(artifact)


class ModelParameters(BaseModel):
    model_name: str

    @property
    def experiment_name(self):
        return self.model_name


async def run_experiment() -> List[ScrapeRow]:
    rows = []
    for url in URL:
        web_content = await ascrape_web(url)
        assert web_content is not None

        print(f'thumbnail_url={web_content.thumbnail}')
        rows.append(
            ScrapeRow(
                url=web_content.url,
                thumbnail_url=web_content.thumbnail,
                title=web_content.title,
                text=web_content.text,
                texts=web_content.texts,
                images=web_content.images,
            )
        )
    return rows


async def run_content_signature_experiment() -> List[ScrapeRow]:
    rows = []
    for url, signature in URL_CONTENT_SIGNATURE:
        s3_path = s3_path_from_signature(signature)
        content_bytes = r2_manager.get_from_s3_path(s3_path)
        if not os.path.exists("/data1/experiments"):
            os.makedirs("/data1/experiments")
        with open(f"/data1/experiments/{signature}.html", "wb") as f:
            f.write(content_bytes)
        web_content = await parse_html(
            html_text=content_bytes.decode("utf-8"), url=url,
        )

        rows.append(
            ScrapeRow(
                url=web_content.url,
                thumbnail_url=web_content.thumbnail,
                title=web_content.title,
                text=web_content.text,
                texts=web_content.texts,
                images=web_content.images,
            )
        )
    return rows


if __name__ == "__main__":
    import wandb
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--experiment_name", required=True, type=str, help="name of the experiment"
    )
    args = parser.parse_args()

    wandb.init(
        # set the wandb project where this run will be logged
        project="blade-web-scraping-v1",
        settings=wandb.Settings(disable_git=True, save_code=False),
        dir="/data1/experiments/",
    )

    combined_df = None
    loop = asyncio.get_event_loop()
    scrapes = loop.run_until_complete(run_experiment())
    content_scrapes = loop.run_until_complete(run_content_signature_experiment())

    render_html(args.experiment_name, scrapes, content_scrapes)
